use std::{collections::HashMap, fmt::{Display, Write}};


#[derive(Debug)]
pub struct Element<'a> {
    pub name: &'a str,
    pub attributes: Option<HashMap<&'a str, &'a str>>,
    pub children: ElBody<'a>,
}

impl<'a> Display for Element<'a> {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(
            f,
            "<{0}{1}>{2}</{0}>",
            self.name,
            if let Some(attrs) = &self.attributes {
                let mut attributes = String::from(" ");

                attrs.iter().for_each(|(key, value)| {
                    attributes
                        .write_fmt(format_args!("{key}=\"{value}\""))
                        .unwrap()
                });

                attributes
            } else {
                "".to_owned()
            },
            self.children
        )
    }
}

/// the direct content of `children`, so it can be a plaintext element
#[derive(Debug)]
pub enum ElBody<'a> {
    Elements(Vec<ElContent<'a>>),
    Text(&'a str),
}
impl<'a> Display for ElBody<'a> {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            ElBody::Text(s) => f.write_str(s),
            ElBody::Elements(els) => els.iter().try_for_each(|e| f.write_str(&e.to_string())),
        }
    }
}

#[derive(Debug)]
pub enum ElContent<'a> {
    El(Element<'a>),
    Text(&'a str),
}

impl<'a> Display for ElContent<'a> {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            ElContent::Text(s) => f.write_str(s),
            ElContent::El(el) => f.write_str(&el.to_string()),
        }
    }
}
